#define _CRT_SECURE_NO_WARNINGS
#include "ExtIO_BB60.h"

#include "bb_api.h"
#pragma comment(lib,"bb_api.lib")

#include <thread>
#include <vector>
#include <iostream>
#include <atomic>

static int handle = -1;
static int iqPerAcquisition = 0;
static int samplesPerSecond = 0;
static double bw = 0.0;
static int64_t center = 1.0e9;

pfnExtIOCallback pfnCallback;

std::vector<float> iq;
std::thread loopThread;

bool stopLoopThread = false;
static std::atomic<bool> reconfigure = false;

struct Atten {
    float displayValue;
};

static const int attenCount = 6;
static int attenIndex = 5;
Atten attenSettings[attenCount] = {
    { 0.0 },
    { 10.0 },
    { 20.0 },
    { 30.0 },
    { 40.0 },
    { 50.0 }
};

void loopRoutine()
{
    stopLoopThread = false;
    reconfigure = true;

    while(!stopLoopThread) {
        if(reconfigure) {
            bbAbort(handle);
            bbConfigureLevel(handle, -attenSettings[attenIndex].displayValue, BB_AUTO_ATTEN);
            bbConfigureCenterSpan(handle, (double)center, 1.0e3);
            bbInitiate(handle, BB_STREAMING, BB_STREAM_IQ);
            reconfigure = false;
        }

        bbIQPacket pkt;
        pkt.iqData = &iq[0];
        pkt.iqCount = iq.size() / 2;
        pkt.triggers = 0;
        pkt.triggerCount = 0;
        pkt.purge = false;
        bbStatus status = bbGetIQ(handle, &pkt);

        extHWstatusT hwStatus = extHw_RUNNING;

        // This didn't do anything when I checked
        //if(status == bbADCOverflow) {
        //    hwStatus = extHw_OVERLOAD;
        //}

        if(status < bbNoError) {
            hwStatus = extHw_ERROR;
        }

        // Send data to HDSDR
        int ret = pfnCallback(iqPerAcquisition, hwStatus, 0, &iq[0]);
    }
}

bool __stdcall InitHW(char *name, char *model, int &hwtype)
{
    sprintf(name, "Signal Hound");
    sprintf(model, "BB60C");
    hwtype = 7;

    bbStatus status = bbOpenDevice(&handle);

    if(status != bbNoError) {
        return false;
    }

    // 40M / decimation
    const int decimation = 2;

    bbConfigureCenterSpan(handle, center, 1.0e3);
    bbConfigureGain(handle, BB_AUTO_GAIN);
    bbConfigureIQ(handle, decimation, 40.0e6 / decimation);
    bbConfigureLevel(handle, -50.0, BB_AUTO_ATTEN);

    bbInitiate(handle, BB_STREAMING, 0);
    bbQueryStreamInfo(handle, &iqPerAcquisition, &bw, &samplesPerSecond);
    // Override IQ per acquitions, must be a multiple of 512
    iqPerAcquisition = 262144 / decimation;
    iq.resize(iqPerAcquisition * 2);

    return true;
}

EXT_API bool __stdcall OpenHW()
{
    //loopThread = std::thread(&loopRoutine);
    pfnCallback(-1, extHw_READY, 0, NULL);

    return true;
}

EXT_API void __stdcall CloseHW()
{
    bbCloseDevice(handle);
}

EXT_API int __stdcall StartHW64(int64_t extLOfreq)
{
    bbStatus status = bbInitiate(handle, BB_STREAMING, BB_STREAM_IQ);
    if(status != bbNoError) {
        return -1;
    }

    stopLoopThread = true;
    if(loopThread.joinable()) {
        loopThread.join();
    }
    bbAbort(handle);
    loopThread = std::thread(&loopRoutine);

    return iqPerAcquisition;
}

EXT_API void __stdcall StopHW()
{
    stopLoopThread = true;
    if(loopThread.joinable()) {
        loopThread.join();
    }
    bbAbort(handle);
}

EXT_API void __stdcall SetCallback(pfnExtIOCallback funcptr)
{
    pfnCallback = funcptr;
}

EXT_API int __stdcall SetHWLO64(int64_t extLOfreq) 
{
    if(extLOfreq < 9.0e3) {
        return -1;
    }

    if(extLOfreq > 6.0e9) {
        return 1;
    }

    center = extLOfreq;
    reconfigure = true;

    return 0;
}


EXT_API int __stdcall GetStatus()
{

    return 0;
}

EXT_API void __stdcall RawDataReady(long samprate, 
                                    void *Ldata, 
                                    void *Rdata, 
                                    int numsamples)
{
    int i = 0;
}

EXT_API long __stdcall GetHWLO()
{
    return center;
}

EXT_API long __stdcall GetHWSR()
{
    return samplesPerSecond;
}

EXT_API int __stdcall GetAttenuators(int idx, float *attenuation)
{
    if(idx >= attenCount) {
        return -1;
    }

    *attenuation = attenSettings[idx].displayValue;
    return 0;
}

EXT_API int __stdcall GetActualAttIdx()
{
    return attenIndex;
}

EXT_API int __stdcall SetAttenuator(int idx)
{
    if(idx >= attenCount || idx < 0) {
        return -1;
    }

    attenIndex = idx;
    reconfigure = true;
    return 0;
}

